////////////////////////////////////////////////////////////////////////////////
// File name: CommonLib.hpp
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// WWW page: http://kserg77.chat.ru/CommonLib.html
// Date: 2006 01 26
//
// Description:   . Inline .
////////////////////////////////////////////////////////////////////////////////

__forceinline BOOL RegClass(
   UINT      style,
   WNDPROC   lpfnWndProc,
   int       cbClsExtra,
   int       cbWndExtra,
   HINSTANCE hInstance,
   HICON     hIcon,
   HCURSOR   hCursor,
   HBRUSH    hbrBackground,
   LPCTSTR   lpszMenuName,
   LPCTSTR   lpszClassName)
{
   WNDCLASS wc = { style,
                   lpfnWndProc,
                   cbClsExtra,
                   cbWndExtra,
                   hInstance,
                   hIcon,
                   hCursor,
                   hbrBackground,
                   lpszMenuName,
                   lpszClassName};
   return (RegisterClass(&wc) != 0);
}

__forceinline BOOL InitializeCriticalSectionEx(CRITICAL_SECTION &cs, DWORD dwSpinCount) {
   BOOL bRes = TRUE;
#if _WIN32_WINNT < 0x0500
   ::InitializeCriticalSection(&cs); //     ...
#else
   DWORD dwErrCode = ERROR_SUCCESS;
   bRes = ::InitializeCriticalSectionAndSpinCount(&cs, dwSpinCount);
   if (!bRes) {
      dwErrCode = ::GetLastError();
      memset(&cs, 0, sizeof(CRITICAL_SECTION));
   }
   ::SetLastError(dwErrCode);
#endif
   return bRes;
}

__forceinline CRITICAL_SECTION InitializeCriticalSectionEx(DWORD dwSpinCount) {
   CRITICAL_SECTION cs;
   InitializeCriticalSectionEx(cs, dwSpinCount);
   return cs;
}

__forceinline CRITICAL_SECTION InitializeCriticalSection() {
   CRITICAL_SECTION cs;
   InitializeCriticalSectionEx(cs);
   return cs;
}

__forceinline RECTEX FindInnerRect(const SIZE &sizeInner, const SIZE &sizeOutward) {
   //   (sizeOutward)   
   //  (sizeInner)      
   //   , ..       .
   //       .
   POINTFLOAT percent = {(float)sizeInner.cx/sizeOutward.cx,
                         (float)sizeInner.cy/sizeOutward.cy};
   RECTEX Rect;
   if (percent.x > percent.y) {
      Rect.right  = sizeOutward.cx;
      Rect.bottom = (long)((float)sizeInner.cy/percent.x);

      Rect.top    = (long)((float)sizeOutward.cy/2-(float)Rect.bottom/2);
      Rect.bottom = (long)((float)sizeOutward.cy/2+(float)Rect.bottom/2);
   } else {
      Rect.right  = (long)((float)sizeInner.cx/percent.y);
      Rect.bottom = sizeOutward.cy;

      Rect.left   = (long)((float)sizeOutward.cx/2-(float)Rect.right/2);
      Rect.right  = (long)((float)sizeOutward.cx/2+(float)Rect.right/2);
   }
   return Rect;
}

__forceinline RECTEX FindInnerRect(const RECT &rectInner, const RECT &rectOutward) {
   return RECTEX(
      FindInnerRect(
         RECTEX(rectInner  ).size(),
         RECTEX(rectOutward).size()
      )
   ).move(rectOutward.left, rectOutward.top);
}

__forceinline BOOL PointInRect(const POINT &point, const RECT &rect) {
   return ::PtInRect(&rect, point);
}

__forceinline BOOL IntersectRect(const RECT &rect1, const RECT &rect2) {
   return ::IntersectRect(&RECTEX(), &rect1, &rect2);
}

__forceinline bool PointInPolygon(const POINT& point, const POINT* const polygon, int size) {
   POINTFLOAT a = {point.x+0.01f, point.y+0.01f};
   int count = 0;
   for (int i=0; i<size; i++) {
      int j = (i+1)%size;
      if (polygon[i].y == polygon[j].y) continue;
      if (polygon[i].y > a.y && polygon[j].y > a.y) continue;
      if (polygon[i].y < a.y && polygon[j].y < a.y) continue;
      if (max(polygon[i].y, polygon[j].y) == a.y) count++;
      else
         if (min(polygon[i].y, polygon[j].y) == a.y) continue;
         else {
            float t = (float)(a.y-polygon[i].y)/(polygon[j].y-polygon[i].y);
            if (t>0 && t<1 && polygon[i].x+t*(polygon[j].x-polygon[i].x) >= a.x) count++;
         }
   }
   return count & 1;
}

__forceinline BOOL MoveWindow(HWND hWnd, const RECT &rect, BOOL bRepaint) {
   return ::MoveWindow(
      hWnd,
      rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
      bRepaint);
}

__forceinline BOOL SetWindowPos(HWND hWnd, HWND hWndInsertAfter, const RECT &rect, UINT uFlags) {
   return ::SetWindowPos(
      hWnd,
      hWndInsertAfter,
      rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top,
      uFlags);
}

__forceinline RECTEX ScreenToClient(HWND hWnd, const RECT &rect) {
   return RECTEX(
      ScreenToClient(hWnd, RECTEX(rect).pointLT()),
      ScreenToClient(hWnd, RECTEX(rect).pointRB())
   );
}

__forceinline RECTEX ClientToScreen(HWND hWnd, const RECT &rect) {
   return RECTEX(
      ClientToScreen(hWnd, RECTEX(rect).pointLT()),
      ClientToScreen(hWnd, RECTEX(rect).pointRB())
   );
}

__forceinline POINTEX ClientToScreen(HWND hWnd, const POINT &point) {
   POINTEX pointResult = point;
   return ::ClientToScreen(hWnd, &pointResult) ? pointResult : POINTEX();
}

__forceinline POINTEX ScreenToClient(HWND hWnd, const POINT &point) {
   POINTEX pointResult = point;
   return ::ScreenToClient(hWnd, &pointResult) ? pointResult : POINTEX();
}

__forceinline RECTEX GetWindowRect(HWND hWnd) {
   RECTEX rect;
   return ::GetWindowRect(hWnd, &rect) ? rect : RECTEX();
}

__forceinline SIZEEX GetWindowSize(HWND hWnd) {
   return (SIZEEX)GetWindowRect(hWnd);
}

__forceinline RECTEX GetClientRect(HWND hWnd) {
   RECTEX rect;
   return ::GetClientRect(hWnd, &rect) ? rect : RECTEX();
}

__forceinline SIZEEX GetClientSize(HWND hWnd) {
   return (SIZEEX)GetClientRect(hWnd);
}

__forceinline POINTEX GetCursorPos() {
   POINTEX pointCur;
   return ::GetCursorPos(&pointCur) ? pointCur : POINTEX();
}

__forceinline void ShowCursor() {
   while (::ShowCursor(TRUE)<0) {}
}

__forceinline void HideCursor() {
   while (::ShowCursor(FALSE)>=0) {}
}

__forceinline CString GetClassName(HWND hWnd) {
   TCHAR szClassName[256] = {0};
   int iRes = ::GetClassName(hWnd, szClassName, 256);
   return iRes ? CString(szClassName) : CString();
}

__forceinline WNDPROC GetWindowProc(HWND hWnd) {
   return (WNDPROC)::GetWindowLong(hWnd, GWL_WNDPROC);
}

//__forceinline DWORD GetWindowStyle(HWND hWnd) {
//   return (DWORD)::GetWindowLong(hWnd, GWL_STYLE);
//}

__forceinline DWORD GetWindowStyleEx(HWND hWnd) {
   return GetWindowExStyle(hWnd);
 //return ::GetWindowLong(hWnd, GWL_EXSTYLE);
}

__forceinline LONG GetWindowUserData(HWND hWnd) {
   return ::GetWindowLong(hWnd, GWL_USERDATA);
}

__forceinline WNDPROC SetWindowProc(HWND hWnd, WNDPROC pWndProc) {
   return SubclassWindow(hWnd, pWndProc);
 //return (WNDPROC)::SetWindowLong(hWnd, GWL_WNDPROC, (LONG)pWndProc);
}

__forceinline DWORD SetWindowStyle(HWND hWnd, DWORD lStyle) {
   return (DWORD)::SetWindowLong(hWnd, GWL_STYLE, (LONG)lStyle);
}

__forceinline DWORD SetWindowStyleEx(HWND hWnd, DWORD lStyleEx) {
   return (DWORD)::SetWindowLong(hWnd, GWL_EXSTYLE, (LONG)lStyleEx);
}

__forceinline LONG SetWindowUserData(HWND hWnd, LONG lUserData) {
   return ::SetWindowLong(hWnd, GWL_USERDATA, lUserData);
}

__forceinline BOOL SetMenuText(HMENU hMenu, UINT uItem, BOOL bByPosition, LPCTSTR szNewText) {
   MENUITEMINFO miiGet = {sizeof(MENUITEMINFO), MIIM_TYPE};
   ::GetMenuItemInfo(hMenu, uItem, bByPosition, &miiGet);

   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE, MFT_STRING | ((miiGet.fType & MFT_RADIOCHECK) ? MFT_RADIOCHECK : 0)};
   mii.dwTypeData = (LPTSTR)szNewText;
   return ::SetMenuItemInfo(hMenu, uItem, bByPosition, &mii);
}

__forceinline SIZEEX GetScreenSize() {
   return SIZEEX(
      ::GetSystemMetrics(SM_CXSCREEN),
      ::GetSystemMetrics(SM_CYSCREEN)
   );
}

//         (     )
__forceinline CString GetFileDir(LPCTSTR szPathAlongWithFileName) {
   CString strResult(szPathAlongWithFileName);
   int iFind = strResult.ReverseFind(_T('\\'));
   if (iFind != -1) {
      strResult = strResult.Left(iFind+1);
   }
   return strResult;
}

//      
__forceinline CString GetFileName(LPCTSTR szPathAlongWithFileName) {
   CString strResult(szPathAlongWithFileName);
   int iFind = strResult.ReverseFind(_T('\\'));
   if (iFind != -1) {
      strResult = strResult.Right(strResult.GetLength()-(iFind+1));
   }
   return strResult;
}

__forceinline BOOL PathExist(LPCTSTR szPath)
{
   //BOOL PathIsDirectory( LPCTSTR pszPath );
   //Header shlwapi.h
   //Import library shlwapi.lib

   DWORD dwAttr = GetFileAttributes(szPath);
   return ((dwAttr != 0xFFFFFFFF) && (dwAttr & FILE_ATTRIBUTE_DIRECTORY));
}

__forceinline BOOL FileExist(LPCTSTR szPath)
{
   DWORD dwAttr = GetFileAttributes(szPath);
   return ((dwAttr != 0xFFFFFFFF) && !(dwAttr & FILE_ATTRIBUTE_DIRECTORY));
}

__forceinline CString GetMenuString(HMENU hMenu, UINT uIDItem, UINT uFlag) {
   TCHAR szName[MAX_PATH] = {0};
   ::GetMenuString(hMenu, uIDItem, szName, MAX_PATH, uFlag);
   return CString(szName);
}

// retrieves the full path and filename for the executable file containing the specified module
__forceinline CString GetModuleFileName(HMODULE hModule) {
   TCHAR szPath[MAX_PATH] = {0};
   ::GetModuleFileName(hModule, szPath, MAX_PATH);
   return CString(szPath);
}

//        (    )
__forceinline CString GetModuleDir(HMODULE hModule) {
   return GetFileDir(GetModuleFileName(hModule));
}

__forceinline CString GetComputerName() {
   DWORD dwSize = MAX_COMPUTERNAME_LENGTH + 1;
   TCHAR szCompName[MAX_COMPUTERNAME_LENGTH + 1] = {0}; szCompName[MAX_COMPUTERNAME_LENGTH] = 0;
   BOOL bRes = ::GetComputerName(szCompName, &dwSize);
   return bRes ? CString(szCompName) : CString();
}

__forceinline CString GetUserName() {
   DWORD dwSize = 256; // == UNLEN is defined in <LMCONS.H>
   TCHAR szUName[256+1] = {0}; szUName[256] = 0;
   BOOL bRes = ::GetUserName(szUName, &dwSize);
   return bRes ? CString(szUName) : CString();
}

__forceinline CString GetCurrentDirectory() {
   DWORD dwNeed = ::GetCurrentDirectory(0, NULL);
   CString strCurrDir;
   DWORD dwErrCode = ERROR_SUCCESS;
   TCHAR *szDir = new TCHAR[dwNeed];
   if (szDir == NULL) {
      dwErrCode = ERROR_OUTOFMEMORY;
   } else {
      szDir[0] = 0;
      dwNeed = ::GetCurrentDirectory(dwNeed, szDir);
      if (dwNeed == 0) {
         dwErrCode = ::GetLastError();
      } else {
         strCurrDir = szDir;
      }
      delete [] szDir; szDir = NULL;
   }
   ::SetLastError(dwErrCode);
   return strCurrDir;
}

__forceinline CString GetSystemDirectory() {
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT uiRes = ::GetSystemDirectory(szDir, MAX_PATH);
   return uiRes ? CString(szDir) : CString();
}

__forceinline CString GetWindowsDirectory() {
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT uiRes = ::GetWindowsDirectory(szDir, MAX_PATH);
   return uiRes ? CString(szDir) : CString();
}

__forceinline CString GetTempPath() {
   //   ,  .  GetTempPath   ?
   // char sz_temp[MAX_PATH] = {0};
   // SHGetFolderPath( 0, CSIDL_INTERNET_CACHE, 0, 0, sz_temp );
   TCHAR szDir[MAX_PATH] = {0}; szDir[MAX_PATH-1] = 0;
   UINT dwRes = ::GetTempPath(MAX_PATH, szDir);
   return dwRes ? CString(szDir) : CString();
}

__forceinline SIZEEX SizeIcon(HICON hIcon) {
   SIZEEX resSize;
   ICONINFO iconInfo;
   if (::GetIconInfo(hIcon, &iconInfo)) {
      resSize.cx = iconInfo.xHotspot<<1;
      resSize.cy = iconInfo.yHotspot<<1;
   }
   return resSize;
}

__forceinline SIZEEX SizeBitmap(HBITMAP hBmp) {
   SIZEEX resultSize;
   BITMAP bmp = {0,0,0,0,0,0,NULL};
   int iRes = ::GetObject(hBmp, sizeof(BITMAP), &bmp);
   if (iRes == sizeof(BITMAP)) {
      resultSize.cx = bmp.bmWidth;
      resultSize.cy = bmp.bmHeight;
   }
   return resultSize;
}

__forceinline SIZEEX SizeBitmap(HDC hCDC) {
   static HBITMAP hBmpNil = ::CreateDiscardableBitmap(hCDC, 1, 1);
   HBITMAP hBmp = (HBITMAP)::SelectObject(hCDC, hBmpNil);
   SIZEEX resultSize(SizeBitmap(hBmp));
   ::SelectObject(hCDC, hBmp);
   //::DeleteObject(hBmpNil);
   return resultSize;
}

__forceinline SIZEEX SizeBitmap(HWND hWnd, BOOL bClientRect) {
   return bClientRect ? ::GetClientSize(hWnd) : ::GetWindowSize(hWnd);
}

__forceinline HBITMAP CreateBitmap(int iWidth, int iHeight) {
   HWND hWnd = ::GetDesktopWindow();
   HDC hDC = ::GetDC(hWnd);
   HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, iWidth, iHeight);
   ::ReleaseDC(hWnd, hDC);
   return hBmp;
}

__forceinline HBITMAP CreateBitmap(const SIZE &size) {
   return ::CreateBitmap(size.cx, size.cy);
}

__forceinline HBITMAP CreateMask(const SIZE &sizeBmp) {
   return ::CreateBitmap(sizeBmp.cx, sizeBmp.cy, 1, 1, NULL);
}

//     0  maxDiapason,   
__forceinline int rand(int maxDiapason) {
   if (maxDiapason < 1) return 0;
   static bool bInitRandom = (srand((unsigned)time(NULL)), true);
   return rand()%(maxDiapason+1);
   return int(float(maxDiapason+1)*rand()/(RAND_MAX+1));
}

__forceinline CString Format(LPCTSTR szFormat, ...) {
   va_list arglist;
   va_start(arglist, szFormat);
   CString strResult;
   strResult.FormatV(szFormat, arglist);
   va_end(arglist);
   return strResult;
}

#ifndef _UNICODE
__forceinline CString AnsiToOem(LPCTSTR szStr) {
   CString str(szStr);
   str.AnsiToOem();
   return str;
}

__forceinline CString OemToAnsi(LPCTSTR szStr) {
   CString str(szStr);
   str.OemToAnsi();
   return str;
}
#endif // _UNICODE

__forceinline CString Replace(IN const CString &str, IN LPCTSTR szOld, IN LPCTSTR szNew) {
   CString strRes = str;
   strRes.Replace(szOld, szNew);
   return strRes;
}

__forceinline CString Replace(IN const CString &str, IN TCHAR cOld, IN TCHAR cNew) {
   CString strRes = str;
   strRes.Replace(cOld, cNew);
   return strRes;
}

__forceinline SYSTEMTIME GetSystemTime() {
   SYSTEMTIME SysTime = {0,0,0,0,0,0,0,0};
   ::GetSystemTime(&SysTime);
   return SysTime;
}

__forceinline SYSTEMTIME GetLocalTime() {
   SYSTEMTIME localTime = {0,0,0,0,0,0,0,0};
   ::GetLocalTime(&localTime);
   return localTime;
}

__forceinline FILETIME GetSystemTimeAsFileTime() {
   FILETIME FileTime = {0,0};
   ::GetSystemTimeAsFileTime(&FileTime);
   return FileTime;
}

__forceinline SYSTEMTIME FileTimeToSystemTime(const FILETIME &FileTime) {
   SYSTEMTIME SysTime = {0,0,0,0,0,0,0,0};
   BOOL bRes = ::FileTimeToSystemTime(&FileTime, &SysTime);
   return SysTime;
}

__forceinline FILETIME SystemTimeToFileTime(const SYSTEMTIME &SysTime) {
   FILETIME FileTime = {0,0};
   BOOL bRes = ::SystemTimeToFileTime(&SysTime, &FileTime);
   return FileTime;
}

__forceinline FILETIME FileTimeToLocalFileTime(const FILETIME &FileTime) {
   FILETIME ftRes = {0,0};
   BOOL bRes = ::FileTimeToLocalFileTime(&FileTime, &ftRes);
   return ftRes;
}

__forceinline BOOL BitBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, sizeSrc.cx, sizeSrc.cy, dwROp);
}

__forceinline BOOL BitBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, 0, 0, dwROp);
}

__forceinline BOOL BitBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, sizeSrc.cx, sizeSrc.cy, dwROp);
}

__forceinline BOOL BitBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, DWORD dwROp) {
   return ::BitBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, 0, 0, dwROp);
}

__forceinline BOOL StretchBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const RECT &rectSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, rectSrc.left, rectSrc.top, rectSrc.right-rectSrc.left, rectSrc.bottom-rectSrc.top, dwROp);
}

__forceinline BOOL StretchBlt(HDC hDCDst, const RECT &rectDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, rectDst.left, rectDst.top, rectDst.right-rectDst.left, rectDst.bottom-rectDst.top, hDCSrc, 0, 0, sizeSrc.cx, sizeSrc.cy, dwROp);
}

__forceinline BOOL StretchBlt(HDC hDCDst, const SIZE &sizeDst, HDC hDCSrc, const SIZE &sizeSrc, DWORD dwROp) {
   return ::StretchBlt(hDCDst, 0, 0, sizeDst.cx, sizeDst.cy, hDCSrc, 0, 0, sizeSrc.cx, sizeSrc.cy, dwROp);
}

__forceinline CString MIMEType2FileType(LPCTSTR szMIMEType) {
   CString strRes = CRegistry::GetString(_T("HKEY_CLASSES_ROOT\\MIME\\Database\\Content Type\\") + CString(szMIMEType) + _T("\\Extension"));
   if (!strRes.IsEmpty()) {
      strRes = strRes.Mid(1);
   }
   return strRes;
}

__forceinline CString FileType2MIMEType(LPCTSTR szFileType) {
   return CRegistry::GetString(_T("HKEY_CLASSES_ROOT\\.") + CString(szFileType) + _T("\\Content Type"));
}

__forceinline CString GUID2String(IN REFGUID guid) {
   CString strRes;
   OLECHAR szRes[39] = {0};
   if (::StringFromGUID2(guid, szRes, sizeof(szRes)>>1)) {
      strRes = szRes;
   } else {
      strRes.Format(_T("{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}"),
         int(guid.Data1), int(guid.Data2), int(guid.Data3), int(guid.Data4[0]), int(guid.Data4[1]), int(guid.Data4[2]), int(guid.Data4[3]), int(guid.Data4[4]), int(guid.Data4[5]), int(guid.Data4[6]), int(guid.Data4[7]));
   }
   return strRes;
}

__forceinline GUID CoCreateGuid() {
   GUID guid; memset(&guid, 0, sizeof(GUID));
   ::CoCreateGuid(&guid);
   return guid;
}

__forceinline GUID String2GUID(IN LPCTSTR szGUID) {
   GUID guid = {0xFFFFFFFF, 0xFFFF, 0xFFFF, {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}};
   String2GUID(szGUID, guid);
   return guid;
}


__forceinline CString Language(WORD wIDLanguage, LCTYPE LCType) {
   TCHAR szRes[64] = {0};
   int iRes = ::GetLocaleInfo(MAKELCID(wIDLanguage, SORT_DEFAULT), LCType, szRes, (sizeof(szRes) << (sizeof(TCHAR)-1)));
   return iRes ? szRes : NULL;
}
